import re
import urllib.parse
from datetime import datetime
from typing import Dict, Final, Generator

import requests
from bs4 import BeautifulSoup
from ythesis import Thesis

from journal_crawler import JournalCrawler
from journal_crawler.const import BROWSER_HEADER
from journal_crawler.utils import parse_authors

BASE_URL: Final[str] = 'https://www.intlpress.com/'
JOURNAL_CONTENTS: Final[Dict[str, str]] = {
    'acta': 'Acta Mathematica',
    'atmp': 'Advances in Theoretical and Mathematical Physics',
    'amsa': 'Annals of Mathematical Sciences and Applications',
    'arkiv': 'Arkiv för Matematik',
    'ajm': 'Asian Journal of Mathematics',
    'cjm': 'Cambridge Journal of Mathematics',
    'cag': 'Communications in Analysis and Geometry',
    'cis': 'Communications in Information and Systems',
    'cms': 'Communications in Mathematical Sciences',
    'cntp': 'Communications in Number Theory and Physics',
    # 'cdm': 'Current Developments in Mathematics',
    # 'dpde': 'Dynamics of Partial Differential Equations',
    # 'gic': 'Geometry, Imaging and Computing',
    # 'hha': 'Homology, Homotopy and Applications',
    # 'joc': 'Journal of Combinatorics',
    # 'jdg': 'Journal of Differential Geometry',
    # 'jsg': 'Journal of Symplectic Geometry',
    # 'mrl': 'Mathematical Research Letters',
    # 'mcgd': 'Mathematics, Computation and Geometry of Data',
    # 'maa': 'Methods and Applications of Analysis',
    # 'iccm': 'Notices of the International Congress of Chinese Mathematicians',
    # 'pamq': 'Pure and Applied Mathematics Quarterly',
    # 'sii': 'Statistics and Its Interface',
    # 'sdg': 'Surveys in Differential Geometry',
}


class InternationalPressCrawler(JournalCrawler):

    def crawl_volumes(self, journal_name: str) -> Dict[str, str]:
        """ from volume title to volume url """
        journal_url = 'https://www.intlpress.com/site/pub/pages/journals/items/' + \
            f'{journal_name}/content/vols/index.php'
        journal_response = requests.get(journal_url, headers=BROWSER_HEADER)
        journal_soup = BeautifulSoup(journal_response.text, 'html.parser')

        volumes: Dict[str, str] = {}
        volumes_elem = journal_soup.find('div', {
            'id': 'mainbody'
        }).find('div', {
            'id': 'list'
        }).find_all('div', {'class': 'list_item'})
        for volume_elem in volumes_elem:
            href = urllib.parse.urljoin(BASE_URL, volume_elem.find('a').get('href'))
            volume_title = volume_elem.find('a').find('p').get_text().strip()
            volumes[volume_title] = href

        return volumes

    def crawl_thesises_one_journal(self, journal_name: str) -> Generator[Thesis, None, None]:
        volumes = self.crawl_volumes(journal_name)
        for volume_title, volume_url in volumes.items():
            print(volume_title)
            published_year = int(re.search(r'\d{4}', volume_title).group())  # type: ignore
            volume_response = requests.get(volume_url, headers=BROWSER_HEADER)
            if volume_response.status_code > 300:
                continue

            volume_soup = BeautifulSoup(volume_response.text, 'html.parser')
            issue_elems = volume_soup.find('div', {
                'id': 'list'
            }).find_all('div', {'class': 'list_item'})

            issues = {}
            for issue_elem in issue_elems:
                issue_title = issue_elem.find('a').find('p').get_text()
                issue_url = issue_elem.find('a').get('href')
                issue_url = urllib.parse.urljoin(BASE_URL, issue_url)
                issues[issue_title] = issue_url

            for issue_title, issue_url in issues.items():
                content_response = requests.get(issue_url, headers=BROWSER_HEADER)
                if content_response.status_code > 300:
                    continue
                content_soup = BeautifulSoup(content_response.text, 'html.parser')
                print(90, issue_title, issue_url)
                no_contents_notice = bool(
                    content_soup.find('div', {
                        'id': 'mainbody'
                    }).find('div', {
                        'id': 'list'
                    }).find('p', {'class': 'nocontentsnotice'}))

                if no_contents_notice:
                    continue

                content_elems = content_soup.find('div', {
                    'id': 'list'
                }).find_all('div', {'class': 'list_item'})

                for content_elem in content_elems:
                    thesis_title = content_elem.find('a').find('p').get_text()

                    if not content_elem.find('p', {'class': 'authors'}):
                        continue

                    thesis_authors = parse_authors(
                        content_elem.find('p', {
                            'class': 'authors'
                        }).get_text())
                    thesis_url = content_elem.find('a').get('href')
                    thesis_url = urllib.parse.urljoin(BASE_URL, thesis_url)
                    yield (
                        Thesis(
                            title=thesis_title,
                            authors=thesis_authors,
                            published=datetime(published_year, 1, 1),
                            journal_name=JOURNAL_CONTENTS[journal_name],
                            url=thesis_url,
                        ))

    def crawl_thesises(self) -> Generator[Thesis, None, None]:
        for journal_name in JOURNAL_CONTENTS:
            print(journal_name)
            yield from self.crawl_thesises_one_journal(journal_name)


def main() -> None:
    crawler = InternationalPressCrawler()
    thesises = list(crawler.crawl_thesises())

    from pprint import pprint
    pprint(thesises)
    print('length', len(thesises))


if __name__ == '__main__':
    main()



from typing import Dict, Final

BROWSER_HEADER: Final[Dict[str, str]] = {
    'User-Agent':
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_3) AppleWebKit/537.36 (KHTML, like Gecko) ' +
        'Chrome/88.0.4324.96 Safari/537.36'
}



from abc import ABCMeta, abstractmethod
from typing import Generator

from ythesis import Thesis


class JournalCrawler:
    __metaclass__ = ABCMeta

    @abstractmethod
    def crawl_thesises(self) -> Generator[Thesis, None, None]:
        ...



import pickle
from pathlib import Path
from typing import Final, Set, Tuple

from ythesis import Thesis

from journal_crawler import JournalCrawler
from journal_crawler.journals.international_press_crawler import InternationalPressCrawler
from journal_crawler.journals.journal_of_singularities_crawler import JournalOfSingularitiesCrawler

THESISES_PATH: Final[Path] = Path(
    Path(__file__).absolute().parent.parent / 'data' / 'thesises.pkl')
CRAWLER_CLASSES: Final[Tuple[JournalCrawler, ...]] = (
    JournalOfSingularitiesCrawler(), InternationalPressCrawler())


def main() -> None:
    thesises: Set[Thesis] = set()
    for crawler in CRAWLER_CLASSES:
        print('crawler:', type(crawler).__name__)
        thesises |= set(crawler.crawl_thesises())

    from pprint import pprint
    pprint(thesises)
    with open(str(THESISES_PATH), 'wb') as f:
        pickle.dump(thesises, f)

    print('length', len(thesises))


if __name__ == '__main__':
    main()



from typing import Tuple


def parse_authors(authors_: str) -> Tuple[str, ...]:
    authors = authors_.strip().replace(' and ', ', ').replace(',and ', ', ').split(',')

    thesis_authors = []
    for author in authors:
        author = author.strip()
        if author == '':
            continue
        thesis_authors.append(author)

    return tuple(thesis_authors)


if __name__ == '__main__':
    assert parse_authors('yassu') == ('yassu',)
    assert parse_authors('yassu, yassu2') == ('yassu', 'yassu2')
    assert parse_authors('yassu, yassu2 and yassu3') == ('yassu', 'yassu2', 'yassu3')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['journal_crawler', 'journal_crawler.journals']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.3,<5.0.0',
 'requests>=2.25.1,<3.0.0',
 'ythesis>=0.1.3,<0.2.0']

setup_kwargs = {
    'name': 'journal-crawler',
    'version': '0.0.1',
    'description': 'provides thesis entities',
    'long_description': 'journal crawler\n================================================================================\n',
    'author': 'yassu',
    'author_email': 'yasu0320.dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/yassu/ythesis',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import re
import urllib.parse
from datetime import datetime
from typing import Dict, Generator, List

import requests
from bs4 import BeautifulSoup
from ythesis import Thesis

from journal_crawler import JournalCrawler
from journal_crawler.const import BROWSER_HEADER
from journal_crawler.utils import parse_authors

BASE_URL = 'https://www.journalofsing.org/'

ROOT_RESPONSE = requests.get('https://www.journalofsing.org/', headers=BROWSER_HEADER)
ROOT_SOUP = BeautifulSoup(ROOT_RESPONSE.text, 'html.parser')

JOURNAL_LIST_SOUP = ROOT_SOUP.find('div', {
    'class': 'box',
    'id': 'col'
}).find('div').find('div').find('ul').find('ul')


class JournalOfSingularitiesCrawler(JournalCrawler):

    def crawl_journal_links(self) -> Dict[str, str]:
        """ get dict from title to url """
        journal_links: Dict[str, str] = {}
        for journal_soup in JOURNAL_LIST_SOUP:
            a_elem = journal_soup.find('a')
            if a_elem == -1 or a_elem is None:
                continue
            href = a_elem.get('href')
            if 'volume' not in href:
                continue

            journal_title = a_elem.get_text()
            journal_links[journal_title] = urllib.parse.urljoin(BASE_URL, href)

        return journal_links

    def crawl_thesises(self) -> Generator[Thesis, None, None]:
        journal_links = self.crawl_journal_links()

        for journal_title, url in journal_links.items():
            published_year = int(re.search(r'\d{4}', journal_title).group())  # type: ignore
            published = datetime(published_year, 1, 1)
            thesises_response = requests.get(url, headers=BROWSER_HEADER)
            thesises_soup = BeautifulSoup(thesises_response.text, 'html.parser')
            thesises_elem = thesises_soup.find('div', {
                'class': 'box',
                'id': 'col'
            }).find('div', {
                'id': 'col-text-content2'
            }).find_all('table')
            for thesis_elem in thesises_elem:
                trs = thesis_elem.find_all('tr')
                if trs[0].find('td').find('img'):
                    continue
                thesis_title = trs[0].find('td').find('a').find('em').get_text().strip()
                thesis_href = trs[0].find('td').find('a').get('href')

                if thesis_title == 'Preface':
                    continue
                authors = parse_authors(trs[1].find('td').get_text())
                yield (
                    Thesis(
                        title=thesis_title,
                        authors=authors,
                        published=published,
                        journal_name='Journal of Singularities',
                        url=urllib.parse.urljoin(url, thesis_href)))


def main() -> None:
    crawler = JournalOfSingularitiesCrawler()
    thesises: List[Thesis] = list(crawler.crawl_thesises())

    from pprint import pprint
    pprint(thesises)
    print('length', len(thesises))


if __name__ == '__main__':
    main()
